/* $Id: $
 *
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <cpl.h>

#include "sph_ldt.h"
#include "sph_dataset.h"
#include "sph_common_keywords.h"
#include <math.h>
#include "sph_error.h"
#include "sph_test.h"


#include "sph_test.h"
#include "sph_utils.h"
#include "sph_test_image_tools.h"

/*----------------------------------------------------------------------------*/
/**
 * @defgroup techcal_master_test  Unit test of techcal_master recipe and
 *                                  associated functions.
 *
 */
/*----------------------------------------------------------------------------*/
/**@{*/

static sph_ldt*
cutest_sph_ldt_helper_create_ldt_standard(void) {
    sph_ifs_lenslet_model* model = NULL;
    cpl_image* inimage = NULL;
    sph_pixel_description_table* pdt = NULL;
    sph_ldt* ldt = NULL;
    int nx = 256;
    int ny = 256;
    int ll = 0;
    sph_lenslet_descriptor* ldescr = NULL;
    cpl_vector* vec = NULL;

    model = sph_test_create_small_lenslet_model();

    inimage = sph_test_image_tools_create_flat_image_double(nx, ny, 0.0);
    pdt = sph_pixel_description_table_new_from_model(model, 0.0, 0.0);
    ldt = sph_ldt_new_from_pdt_image(pdt, model, inimage);
    sph_pixel_description_table_delete(pdt);
    cpl_image_delete(inimage);
    vec = cpl_vector_duplicate(ldt->reflambdas);
    for (ll = 0; ll < ldt->nlens; ++ll) {
        ldescr = ldt->arr[ll];
        if (ldescr) {
            cpl_vector_fill(vec, 2.0);
            sph_lenslet_descriptor_fill_values(ldescr, vec);
            cpl_vector_fill(vec, 0.0);
            sph_lenslet_descriptor_fill_bpix(ldescr, vec);
            cpl_vector_fill(vec, 3.0);
            sph_lenslet_descriptor_fill_rms(ldescr, vec);
            cpl_vector_fill(vec, 4.0);
            sph_lenslet_descriptor_fill_ncomb(ldescr, vec);
        }
    }

    cpl_vector_delete(vec);
    vec = NULL;
    return ldt;
}

static
int cutest_init_testsuite(void) {
    /*--------------------------------------------------------------------
     * -    Prepare CPL and error logging
     * -------------------------------------------------------------------*/
    sph_test_nop_code();
    return 0;
}
static
int cutest_clean_testsuite(void) {
    return sph_end_test();
}

static
void cutest_standard_happy_case(void) {
    sph_master_frame* mframe = NULL;
    cpl_propertylist* plist = NULL;
    sph_ldt* ldt = NULL;

    ldt = cutest_sph_ldt_helper_create_ldt_standard();
    cpl_test_nonnull( ldt );

    mframe = sph_ldt_collapse(ldt, 0.0, 0.5);

    sph_master_frame_save(mframe, "cutest_sph_ldt_standard_happy_case.fits",
            NULL);

    cpl_test_abs(cpl_image_get_flux(mframe->image),
            ldt->nlens * 2.0 * cpl_vector_get_size(ldt->reflambdas), 1.0);
    sph_error_raise(SPH_ERROR_GENERAL, __FILE__, __func__, __LINE__,
            SPH_ERROR_INFO, "flux:%f vs. %f\n",
            cpl_image_get_flux(mframe->image),
            ldt->nlens * 2.0 * cpl_vector_get_size(ldt->reflambdas));
    sph_master_frame_delete(mframe);
    cpl_propertylist_delete(plist);
    plist = NULL;

    sph_ldt_delete(ldt);
}
#ifdef SPHERE_TEST_FULL
static
void cutest_big_happy_case(void) {
    sph_master_frame* mframe = NULL;
    int nx = 402;
    int ny = 402;
    int xx = 0;
    int yy = 0;
    int bad = 0;
    int bp = 0;
    sph_ldt* ldt = create_ldt_big();
    cpl_propertylist* plist = NULL;
    cpl_test_nonnull( ldt );
    mframe = sph_ldt_create_mframe(ldt,nx,ny,1,0.0);
    cpl_test_nonnull( mframe );
    cpl_test_eq(
            cpl_image_save( mframe->image, "bigimage.fits", CPL_TYPE_DOUBLE,
                    plist = sph_ifs_lenslet_model_get_as_propertylist( ldt->model ),
                    CPL_IO_CREATE ), CPL_ERROR_NONE);
    cpl_test_abs( cpl_image_get_flux(mframe->image),ldt->nlens * 2.0, 1.0 );
    sph_master_frame_delete(mframe);
    cpl_propertylist_delete(plist); plist = NULL;
    sph_ldt_delete(ldt);
}
#endif
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit tests of techcal_master_dark recipe and associated functions
 */
/*----------------------------------------------------------------------------*/
int main(void) {
    int result = 0;
    const void* pSuite = NULL;


    if ( 0 != sph_test_init())
        return sph_test_get_error();


    pSuite = sph_add_suite("ldt_create_image", cutest_init_testsuite,
            cutest_clean_testsuite);
    if (NULL == pSuite) {
        return sph_test_get_error();
    }


    if (NULL
            == sph_test_do(pSuite, "standard happy case",
                    cutest_standard_happy_case)) {
        return sph_test_get_error();
    }

#ifdef SPHERE_TEST_FULL
    if ( NULL == sph_test_do(pSuite, "big happy case",
                    cutest_big_happy_case ) )
    {
        return sph_test_get_error();
    }
#endif

    /* Run all tests using the CUnit Basic interface */
    sph_test_nop_int( 0);
    sph_test_nop_char("results.txt");
    result = sph_test_end();
    return result;
}

/**@}*/
